@@ -6,11 +6,9 @@ import traceback
 
 from .types import Param
 
-__version__ = "0.1.0"
-
 LOGGER = logging.getLogger(__name__)
 
-ENV_ENTRYPOINT = "EP"
+ENV_ENGINE = "MINION_ENGINE"
 
 
 class Engine:
@@ -26,43 +24,47 @@ class Engine:
         pass
 
 
-class Callback(ABC):
+class Callback:
     """Base class for the (user defined) callbacks."""
 
-    @abstractmethod
-    def on_param_received(self, param: Mapping[Any, Any], engine: Engine):
+    def setup(self, param: Param, engine: Engine):
         pass
 
-    @abstractmethod
-    def on_param_finished(
-        self, param: Mapping[Any, Any], engine: Engine, exception: Exception
-    ):
+    def process(self, param: Param, engine: Engine):
         pass
 
+    def teardown(self, param: Param, engine: Engine, exception: Exception):
+        pass
+
+    def finalize(self, param: Param, engine: Engine, exception: Exception):
+        pass
 
 class PrintCallback(Callback):
     """Dummy callback for debug purpose."""
 
-    def on_param_received(self, param, engine):
-        print(f"received {param}")
+    def pre(self, param, engine):
+        print(f"pre {param}")
 
-    def on_param_finished(self, param, engine, exception):
+    def process(self, param, engine):
+        print(f"process {param}")
+        engine.done(param)
+
+    def teardown(self, param, engine, exception):
         print(f"finished {param}")
         # everything gonna be alright
-        engine.done(param)
 
 
 class ProcessCallback(Callback):
     def to_cmd(self, param: Param):
         return f"echo {param}"
 
-    def on_param_received(self, param, engine):
+    def process(self, param: Param, engine: Engine):
         import subprocess
 
         p = subprocess.run(self.to_cmd(param), shell=True, check=True)
         p.check_returncode()
 
-    def on_param_finished(self, param, engine, exception):
+    def finalize(self, param: Param, engine: Engine, exception: Exception):
         if exception is None:
             engine.done(param)
         else:
@@ -83,18 +85,19 @@ class EntryPoint:
             if param is None:
                 LOGGER.debug("No more params to handle, leaving")
                 break
-            LOGGER.debug(f"SWEEP {param}")
+            self.cb.setup(param, self.engine)
             try:
-                self.cb.on_param_received(param, self.engine)
+                self.cb.process(param, self.engine)
             except Exception as e:
                 exception = e
                 traceback.print_exc()
-            self.cb.on_param_finished(param, self.engine, exception)
+            self.cb.finalize(param, self.engine, exception)
+            self.cb.teardown(param, self.engine, exception)
 
 
 def minionizer(callback: Callback):
     # NOTE(msimonin): load dynamically from the name
-    ep = os.getenv(ENV_ENTRYPOINT)
+    ep = os.getenv(ENV_ENGINE)
     if ep == "execo":
         from minionize.execo import Execo
 
